define(function (require, exports, module) {
	'use strict';

	var LanguageManager = brackets.getModule("language/LanguageManager");
	var ExtensionUtils = brackets.getModule("utils/ExtensionUtils");

    CodeMirror.defineMode("xmlconverter", function () 
    {
        return {
            token: function (stream, state) 
            {

               	var endfound = false;

                if ((stream.sol())  && (state.tokentonextline)) 
                {
                	// Fortsetzung eines Tokens
                
                    state.tokentonextline = false;

               		if (state.tokentype === "blockcomment")
                	{
                     	state.tokentype = null;
               			endfound = false;
                		while (!endfound)
                		{
                			if (stream.skipTo("*"))
                			{
                				if (stream.match(/^\*\//))
                				{
                					endfound = true;
                				}
                				else
                				{
                					stream.next();
                				}
                			}
                			else if (stream.skipToEnd())
                			{
                				endfound = true;
                				state.tokentonextline = true;
		                 		state.tokentype = "blockcomment";
                			}
                			else
                			{
                				endfound = true;
                				state.tokentonextline = true;
		                 		state.tokentype = "blockcomment";
                			}
                		}
                    	return "comment";
                	} 
                    else if (state.tokentype === "doublequote") 
                    {
                     	state.tokentype = null;
                        stream.eatWhile(/[^\"]/);
                        if (!stream.eat('"')) 
                        {
                            state.tokentonextline = true;
                            state.tokentype = "doublequote";
                        } 
                        else 
                        {
                            state.tokentonextline = false;
                        }
                    	return "quote";
                    } 
                    else if (state.tokentype === "singlequote") 
                    {
                     	state.tokentype = null;
                        stream.eatWhile(/[^\']/);
                        if (!stream.eat("'")) 
                        {
                            state.tokentonextline = true;
                            state.tokentype = "singlequote";
                        } 
                        else 
                        {
                            state.tokentonextline = false;
                		}
                    	return "quote";
                    } 
                    else 
                    {
                        state.tokentype = null;
                        return null;
                    }
                } 
                else 
                {
                	// Suche nach neuen Tokens
                
                	state.tokentonextline = false;
                	
                	if (stream.match(/^\ /)) 
                	{
                		// Leerraum, nix
                    	state.tokentype = null;
                    	return null;
                	} 
                	else if (stream.match(/^\/\//)) 
                	{
                		// Kommentarzeile
                		stream.skipToEnd();
                     	state.tokentype = null;
                    	return "comment";
                	} 
                	else if (stream.match(/^\/\*/)) 
                	{
                		// Kommentarblock, Beginn
                		state.tokentype = null;
                		endfound = false;
                		while (!endfound)
                		{
                			if (stream.skipTo("*"))
                			{
                				if (stream.match(/^\*\//))
                				{
                					endfound = true;
                				}
                				else
                				{
                					stream.next();
                				}
                			}
                			else if (stream.skipToEnd())
                			{
                				endfound = true;
                				state.tokentonextline = true;
		                     	state.tokentype = "blockcomment";
                			}
                			else
                			{
                				endfound = true;
                				state.tokentonextline = true;
		                     	state.tokentype = "blockcomment";
                			}
                		}
                    	return "comment";
                	} 
                	else if (stream.match(/^\"/)) 
                	{
                		// Literal
                		state.tokentype = null;
                		stream.eatWhile(/[^\"]/);
               			if (stream.eat('"')) 
               			{
                			state.tokentonextline = false;
                		} 
                		else 
                		{
                			state.tokentonextline = true;
	                     	state.tokentype = "doublequote";
                		}
                    	return "quote";
                	} 
                	else if (stream.match(/^\'/)) 
                	{
                		// alternatives Literal 
                		state.tokentype = null;
                		stream.eatWhile(/[^\']/);
                    	if (stream.eat("'")) 
                    	{
                			state.tokentonextline = false;
                		} 
                		else 
                		{
                			state.tokentonextline = true;
	                     	state.tokentype = "singlequote";
                		}
                    	return "quote";
                    } 
                    else if (stream.match(/^(ELEMENT)(\s|\ |\t)([A-Za-z][A-Za-z0-9\.\-\_]*)/,false))
                    {
                    	stream.match(/^(ELEMENT)/,true);
                		state.tokentype = null;
                    	return "handler";
                    }
                    else if (stream.match(/^(ENTITY)(\s|\ |\t)([A-Za-z][A-Za-z0-9\.\-\_]*|\#([0-9]+|[A-Z]+)|\#x([0-9A-Fa-f]+))(\ |\t)*\=/,false))
                    {
	                    stream.match(/^(ENTITY)/,true);
                		state.tokentype = null;
                    	return "handler";
                    }
                    else if (stream.match(/^(ENDE)(\s|\ |\t|$)/,false))
                    {
	                    stream.match(/^(ENDE)/,true);
                		state.tokentype = null;
                    	return "handler";
                    }
                    else if (stream.match(/^(ELEMENT|ENTITY)(\s|\ |\t|$)/,false))
                    {
	                    stream.match(/^(ELEMENT|ENTITY)/,true);
                		state.tokentype = null;
                    	return "error";
                    }
                    else if (stream.match(/^(BASE|CONVERTELEMENTS|CONVERTENTITIES|CONVERTDATA|MODEL|READERENCODING|SYNTAX|SYSTEM|WRITERENCODING|WARNINGS)(\s|\ |\t)*\=/,false))
                    {
                    	stream.match(/^(BASE|CONVERTELEMENTS|CONVERTENTITIES|CONVERTDATA|MODEL|READERENCODING|SYNTAX|SYSTEM|WRITERENCODING|WARNINGS)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(BASE|CONVERTELEMENTS|CONVERTENTITIES|CONVERTDATA|MODEL|READERENCODING|SYNTAX|SYSTEM|WRITERENCODING|WARNINGS)(\s|\ |\t|$)/,false))
                    {
                    	stream.match(/^(BASE|CONVERTELEMENTS|CONVERTENTITIES|CONVERTDATA|MODEL|READERENCODING|SYNTAX|SYSTEM|WRITERENCODING|WARNINGS)/,true);
                		state.tokentype = null;
                    	return "error";
                    }
                    else if (stream.match(/^(COPY|COUNT|CHR|CUT|DEL|FILE|FIND|IMPLIED|SUB|VALUE)[\(]/,false))
                    {
	                    stream.match(/^(COPY|COUNT|CHR|CUT|DEL|FILE|FIND|IMPLIED|SUB|VALUE)/,true);
                		state.tokentype = null;
                    	return "function";
                    }
                    else if (stream.match(/^(COPY|COUNT|CHR|CUT|DEL|FILE|FIND|IMPLIED|SUB|VALUE)(\s|\ |\t|$)/,false))
                    {
	                    stream.match(/^(COPY|COUNT|CHR|CUT|DEL|FILE|FIND|IMPLIED|SUB|VALUE)/,true);
                		state.tokentype = null;
                    	return "error";
                    }
                    else if (stream.match(/^(AFTER|AFTERX|BEFORE|BEFOREX|EMPTY|FIRST|LAST|IN|INX|TOPLEVEL)(\s|\ |\t|$)/,false))
                    {
                    	stream.match(/^(AFTER|AFTERX|BEFORE|BEFOREX|EMPTY|FIRST|LAST|IN|INX|TOPLEVEL)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(NOTAFTER|NOTAFTERX|NOTBEFORE|NOTBEFOREX|NOTEMPTY|NOTFIND|NOTLAST|NOTFIRST|NOTIN|NOTINX|NOTSUB|NOTTOPLEVEL)(\s|\ |\t|$)/,false))
                    {
                    	stream.match(/^(NOTAFTER|NOTAFTERX|NOTBEFORE|NOTBEFOREX|NOTEMPTY|NOTFIND|NOTLAST|NOTFIRST|NOTIN|NOTINX|NOTSUB|NOTTOPLEVEL)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(ALTERNATE|ATTLIST)(\s|\ |\t|$)/,false))
                    {
                    	stream.match(/^(ALTERNATE|ATTLIST)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(BEG|END|ERR|DAT)(\s|\ |\t)*\=/,false))
                    {
                    	stream.match(/^(BEG|END|ERR|DAT)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(BEG|END|ERR|DAT)(\s|\ |\t|$)/,false))
                    {
                    	stream.match(/^(BEG|END|ERR|DAT)/,true);
                		state.tokentype = null;
                    	return "error";
                    }
                    else if (stream.match(/^(IF|ELSE|ELSEIF|ENDIF)(\s|\ |\t|$)/,false))
                    {
	                    stream.match(/^(IF|ELSE|ELSEIF|ENDIF)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(DROP|DROPUNDEFINED)(\s|\ |\t|$)/,false))
                    {
                    	stream.match(/^(DROP|DROPUNDEFINED)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(ALL|ALPHA|ALT|ASC|CHILD|CONTENT|DOC|FULL|GROUP|LEVEL)(\s|\ |\,|\)|\t|$)/,false))
                    {
                    	stream.match(/^(ALL|ALPHA|ALT|ASC|CHILD|CONTENT|DOC|FULL|GROUP|LEVEL)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(NO|NUM|ONE|ONLYDEFINED|PARENT|ROM|SGML|SMART|UPALPHA|UPROM|XML|YES)(\s|\ |\,|\)|\t|$)/,false))
                    {
	                    stream.match(/^(NO|NUM|ONE|ONLYDEFINED|PARENT|ROM|SGML|SMART|UPALPHA|UPROM|XML|YES)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^(ASCII|DATA|MIXED|NAMECASE|PUBLIC|TOKEN)(\s|\ |\,|\)|\t|$)/,false))
                    {
	                   	stream.match(/^(ASCII|DATA|MIXED|NAMECASE|PUBLIC|TOKEN)/,true);
                		state.tokentype = null;
                    	return "keyword";
                    }
                    else if (stream.match(/^[\-\+\=\!\<\>]+/,true))
                    {
                		state.tokentype = null;
                    	return "operator";
                    }
                    else 
                    {
                    	// alles andere
                		stream.next();
                		state.tokentype = null;
                		return null;
                	}
				}
            },

            startState: function () 
            {
                return {
                    tokentype: "",
                    tokentonextline: false
                };
            }
        };
    });

	ExtensionUtils.loadStyleSheet(module,"main.css");

    LanguageManager.defineLanguage("xmlconverter", 
    {
        name: "XB",
        mode: "xmlconverter",
        fileExtensions: ["xb"],
        fileNames: [".xb"],
        blockComment: ["/*","*/"],
        lineComment: ["//"]
    });

});